@@ -37,10 +37,10 @@ def index():
         '''
 
 def logout_view():
-    user = logout()
-    if user is None:
+    user_data = logout()
+    if user_data is None:
         return 'No user to log out.'
-    return 'Logged out user {0}.'.format(user.username)
+    return 'Logged out user {0}.'.format(user_data['username'])
 
 # URLs
 app.add_url_rule('/', 'index', index, methods=['GET', 'POST'])
