@@ -25,20 +25,21 @@ class Auth(object):
     Extension initialization object containing settings for the extension.
     
     Supported settings:
-        - login_url_name: Name of the URL that is used for login. It's used in
-        the not_logged_in_callback if provided in the constructor.
-        - not_logged_in_callback: Function to call when a user accesses a page
-        without being logged in. Normally used to redirect to the login page.
-        If a login_url_name is provided, it will by default redirect to that
-        url. Otherwise, the default is abort(401).
-        - not_permitted_callback: Function to call when a user tries to access
-        a page for which he doesn't have the permission. Default: abort(401).
-        - hash_algorithm: Algorithm from the hashlib library used for password
-        encryption. Default: sha1.
-        - user_timeout: Timeout (in seconds) after which the sesion of the user
-        expires. Default: 3600. A timeout of 0 means it will never expire.
-        - load_role: Function to load a role. Is called with user.role as only
-        parameter.
+
+    - login_url_name: Name of the URL that is used for login. It's used in
+      the not_logged_in_callback if provided in the constructor.
+    - not_logged_in_callback: Function to call when a user accesses a page
+      without being logged in. Normally used to redirect to the login page.
+      If a login_url_name is provided, it will by default redirect to that
+      url. Otherwise, the default is abort(401).
+    - not_permitted_callback: Function to call when a user tries to access
+      a page for which he doesn't have the permission. Default: abort(401).
+    - hash_algorithm: Algorithm from the hashlib library used for password
+      encryption. Default: sha1.
+    - user_timeout: Timeout (in seconds) after which the sesion of the user
+      expires. Default: 3600. A timeout of 0 means it will never expire.
+    - load_role: Function to load a role. Is called with user.role as only
+      parameter.
     """
 
     def __init__(self, app=None, login_url_name=None):
@@ -59,23 +60,19 @@ class Auth(object):
         
 class AuthUser(object):
     """
-    Baseclass for a user model. Contains a few convenient methods.
+    Baseclass for a user model. Contains a few convenience methods.
 
     Attributes:
-        - username:
-            Username of the user.
-        - password: 
-            Password of the user. By default not encrypted. The
-            set_and_encrypt_password() method sets and encrypts the password.
-        - salt: 
-            Salt used for the encrytion of the password.
-        - role:
-            Role of this user.
-    """
+
+    - username: Username of the user.
+    - password: Password of the user. By default not encrypted. The 
+      set_and_encrypt_password() method sets and encrypts the password.
+    - salt: Salt used for the encrytion of the password.
+    - role: Role of this user.  """
 
     role = None
 
-    def __init__(self, username, password=None, salt=None, role=None):
+    def __init__(self, username=None, password=None, salt=None, role=None):
         self.username = username
         # Storing password unmodified. Encryption of the password should 
         # happen explicitly.
@@ -102,10 +99,29 @@ class AuthUser(object):
         return False
 
     def __eq__(self, other):
-        return hasattr(other, 'username') and self.username == other.username
+        return self.username == getattr(other, 'username', None)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __getstate__(self):
+        return self.__dict__
+
+    @classmethod
+    def load_current_user(cls, apply_timeout=True):
+        """
+        Load current user based on the result of get_current_user_data().
+        """
+        data = get_current_user_data(apply_timeout)
+        if not data:
+            return None
+        user = cls()
+        user.__dict__ = data
+        return user
 
     def is_logged_in(self):
-        return get_current_user() == self
+        user_data = get_current_user_data()
+        return user_data is not None and user_data.get('username') == self.username
 
 def encrypt(password, salt=None, hash_algorithm=None):
     """Encrypts a password based on the hashing algorithm."""
@@ -121,24 +137,25 @@ def login(user):
     Logs the user in. Note that NO AUTHENTICATION is done by this function. If
     you want to authenticate a user, use the AuthUser.authenticate() method.
     """
-    session[SESSION_USER_KEY] = user
+    session[SESSION_USER_KEY] = user.__getstate__()
     session[SESSION_LOGIN_KEY] = datetime.datetime.utcnow()
 
 def logout():
-    """Logs the currently logged in user out and returns the user (if any)."""
+    """Logs the currently logged in user out and returns the user data."""
     session.pop(SESSION_LOGIN_KEY, None)
     return session.pop(SESSION_USER_KEY, None)
     
-def get_current_user(apply_timeout=True):
+def get_current_user_data(apply_timeout=True):
+    """ 
+    Returns the data of the current user (user.__dict__) if there is a
+    current user and he didn't time out yet. If timeout should be ignored,
+    provide apply_timeout=False.  
     """
-    Returns the current user if there is one and he didn't time out yet. If
-    timeout should be ignored, provide apply_timeout=False.
-    """
-    user = session.get(SESSION_USER_KEY, None)
-    if user is None:
-        return None
+    user_data = session.get(SESSION_USER_KEY, None)
+    if user_data is None:
+        return None 
     if not apply_timeout:
-        return user
+        return user_data
     login_datetime = session[SESSION_LOGIN_KEY]
     now = datetime.datetime.utcnow()
     user_timeout = current_app.auth.user_timeout
@@ -146,9 +163,12 @@ def get_current_user(apply_timeout=True):
        datetime.timedelta(seconds=user_timeout):
         logout()
         return None
-    return user
+    return user_data
 
-def _not_logged_in(callback, *args, **kwargs):
+def not_logged_in(callback, *args, **kwargs):
+    """
+    Executes not logged in callback. Not for external use.
+    """
     if callback is None:
         return current_app.auth.not_logged_in_callback(*args, **kwargs)
     else:
@@ -161,8 +181,8 @@ def login_required(callback=None):
     """
     def wrap(func):
         def decorator(*args, **kwargs):
-            if get_current_user() is None:
-                return _not_logged_in(callback, *args, **kwargs)
+            if get_current_user_data() is None:
+                return not_logged_in(callback, *args, **kwargs)
             return func(*args, **kwargs)
         return decorator
     return wrap
