@@ -3,12 +3,11 @@ Module containing functions and classes specific to the permission model.
 """
 
 from flask import current_app
-from flaskext.auth.auth import get_current_user, _not_logged_in
-user_permissions = dict()
+from flaskext.auth.auth import get_current_user_data, not_logged_in
 
-def has_permission(user, resource, action):
+def has_permission(role, resource, action):
     """Function to check if a user has the specified permission."""
-    role = current_app.auth.load_role(user.role)
+    role = current_app.auth.load_role(role)
     return role.has_permission(resource, action) if role else False
 
 def permission_required(resource, action, callback=None):
@@ -18,9 +17,10 @@ def permission_required(resource, action, callback=None):
     """
     def wrap(func):
         def decorator(*args, **kwargs):
-            if get_current_user() is None:
-                return _not_logged_in(callback, *args, **kwargs)
-            if not has_permission(get_current_user(), resource, action):
+            user_data = get_current_user_data()
+            if user_data is None:
+                return not_logged_in(callback, *args, **kwargs)
+            if not has_permission(user_data.get('role'), resource, action):
                 if callback is None:
                     return current_app.auth.not_permitted_callback(*args, **kwargs)
                 else:
@@ -35,14 +35,16 @@ class Permission(object):
     Permission object, representing actions that can be taken on a resource.
     
     Attributes:
-        - resource: A resource is a component on which actions can be 
-        performed. Examples: post, user, ticket, product, but also 
-        post.comment, user.role, etc.
-        - action: Any action that can be performed on a resource. Names of 
-        actions should be short and clear. Examples: create, read,
-        update, delete, download, list, etc.
+
+    - resource: A resource is a component on which actions can be performed.
+      Examples: post, user, ticket, product, but also post.comment, user.role,
+      etc.
+    - action: Any action that can be performed on a resource. Names of actions
+      should be short and clear. Examples: create, read, update, delete, download,
+      list, etc.
     """
-    def __init__(self, resource, action):
+    
+    def __init__(self, resource, action): 
         self.resource = resource
         self.action = action
 
@@ -54,8 +56,9 @@ class Role(object):
     Role object to group users and permissions.
 
     Attributes:
-        - name: The name of the role.
-        - permissions: A list of permissions.
+
+    - name: The name of the role.
+    - permissions: A list of permissions.
     """
     def __init__(self, name, permissions):
         self.name = name
